package ink.wgink.module.wechat.service.pay.v3.jsapi.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.wechat.enums.PayAuthorizationTypeEnum;
import ink.wgink.module.wechat.pojo.pay.v3.jsapi.pojo.PayPlaceOrder;
import ink.wgink.module.wechat.request.pay.v3.jsapi.PlaceOrderPayRequestImpl;
import ink.wgink.module.wechat.pojo.pay.v3.jsapi.pojo.PayPrepay;
import ink.wgink.module.wechat.service.pay.v3.jsapi.IJsapiService;
import ink.wgink.module.wechat.utils.pay.PayCertificateUtil;
import ink.wgink.properties.wechat.pay.v3.PayProperties;
import ink.wgink.util.BeanPropertyCheckUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestMethod;

import java.io.FileInputStream;
import java.security.cert.X509Certificate;

/**
 * @ClassName: JsapiServiceImpl
 * @Description: Jsapi业务
 * @Author: wanggeng
 * @Date: 2021/8/19 10:42 上午
 * @Version: 1.0
 */
@Service
public class JsapiServiceImpl extends DefaultBaseService implements IJsapiService {

    @Autowired
    private PayProperties payProperties;

    @Override
    public String placeOrder(PayPlaceOrder payPlaceOrder) throws Exception {
        BeanPropertyCheckUtil.checkField(payPlaceOrder);
        String urlSuffix = "/v3/pay/transactions/jsapi";
        String url = "https://api.mch.weixin.qq.com" + urlSuffix;

        // 获得证书
        X509Certificate certificate = PayCertificateUtil.getCertificate(new FileInputStream(payProperties.getCertificatePath()));
        String serialNumber = certificate.getSerialNumber().toString(16);

        PlaceOrderPayRequestImpl placeOrderPayRequest = new PlaceOrderPayRequestImpl();
        String body = placeOrderPayRequest.bodyJsonString(payPlaceOrder);
        String authorization = placeOrderPayRequest.getAuthorization(RequestMethod.POST, urlSuffix, serialNumber, payProperties, body, PayAuthorizationTypeEnum.WECHATPAY2_SHA256_RSA2048);
        // 请求获得结果
        PayPrepay payPrepay = placeOrderPayRequest.post(url, authorization, serialNumber, body);
        return payPrepay.getPrepayId();
    }
}
