package ink.wgink.module.wechat.utils.pay;

import org.bouncycastle.util.encoders.Base64;

import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.security.PublicKey;
import java.security.cert.X509Certificate;

/**
 * @ClassName: PayVerifyUtil
 * @Description: 验证工具
 * @Author: wanggeng
 * @Date: 2021/8/18 11:20 下午
 * @Version: 1.0
 */
public class PayVerifyUtil {

    /**
     * 验证签名
     *
     * @param signature
     * @param body
     * @param nonce
     * @param timestamp
     * @param certInputStream
     * @return
     * @throws Exception
     */
    public static boolean verifySignature(String signature, String body, String nonce, String timestamp, InputStream certInputStream) throws Exception {
        String buildSignMessage = buildSignMessage(timestamp, nonce, body);
        // 获取证书
        X509Certificate certificate = PayCertificateUtil.getCertificate(certInputStream);
        PublicKey publicKey = certificate.getPublicKey();
        return checkByPublicKey(buildSignMessage, signature, publicKey);
    }

    public static boolean checkByPublicKey(String data, String sign, PublicKey publicKey) throws Exception {
        java.security.Signature signature = java.security.Signature.getInstance("SHA256WithRSA");
        signature.initVerify(publicKey);
        signature.update(data.getBytes(StandardCharsets.UTF_8));
        return signature.verify(Base64.decode(sign.getBytes(StandardCharsets.UTF_8)));
    }

    public static String buildSignMessage(String timestamp, String nonceStr, String body) {
        return new StringBuilder()
                .append(timestamp)
                .append("\n")
                .append(nonceStr)
                .append("\n")
                .append(body)
                .append("\n")
                .toString();
    }

}
